<?php
namespace Teknisa\EST\Modules\Relatorios\Controllers;

use Teknisa\EST\Controllers\TeknisaCrud;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Teknisa\EST\Modules\Relatorios\Services\Est46000EmissaoAjusteInventarioService;
use Zeedhi\Framework\Report\ReportService;


class Est46000EmissaoAjusteInventarioController extends \Zeedhi\Framework\Controller\Report{

	protected $novoCodigo;
 	protected $serviceMoviEst;
 	protected $reportService;
 	protected $birtStrategy;
 	protected $reportMapping = array (
        'EST46000' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        "SEQUMOVI"     => "P_SEQUMOVI",
                        "CDFILIAL"     => "P_CDFILIAL",
                        "DTPERINI"     => "P_DTINICIAL",
                        "DTPERFIN"     => "P_DTFINAL",
                        "NRORG"        => "P_NRORG",
                        "CUSTOS"       => "P_CUSTO",
                        "CDALMOXARIFE" => "P_CDALMOXARIFE",
                        "CDLOCALESTOQ" => "P_CDLOCALESTOQ",
                        "NRLOTEESTOQ"  => "P_NRLOTEESTOQ",
                        "NPTOTALIZA"   => "P_NPTOTALIZA",
                        "CDPRODINI"    => "P_CDPRODINI",
                        "CDPRODFIN"    => "P_CDPRODFIN",
                        "ORDENANOME"   => "P_ORDENANOME",
                        "CDFILIAL"	   => "P_CDFILIAL",
                        "IDLANCTO"	   => "P_IDLANCTO"
                    )
            ),
        'EST46000_retirada' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        "SEQUMOVI"     => "P_SEQUMOVI",
                        "CDFILIAL"     => "P_CDFILIAL",
                        "DTPERINI"     => "P_DTINICIAL",
                        "DTPERFIN"     => "P_DTFINAL",
                        "NRORG"        => "P_NRORG",
                        "CUSTOS"       => "P_CUSTO",
                        "CDALMOXARIFE" => "P_CDALMOXARIFE",
                        "CDLOCALESTOQ" => "P_CDLOCALESTOQ",
                        "NRLOTEESTOQ"  => "P_NRLOTEESTOQ",
                        "NPTOTALIZA"   => "P_NPTOTALIZA",
                        "CDPRODINI"    => "P_CDPRODINI",
                        "CDPRODFIN"    => "P_CDPRODFIN",
                        "ORDENANOME"   => "P_ORDENANOME",
                        "CDFILIAL"	   => "P_CDFILIAL",
                        "IDLANCTO"	   => "P_IDLANCTO"
                    )
            )
    );

    public function __construct ($crudService, ParameterBag $parameterBag, Environment $environment, Manager $dataSourceManager, NewCode $newCode, Est46000EmissaoAjusteInventarioService $serviceMoviEst, ReportService $reportService, $birtStrategy) {
        $this->newCode = $newCode;
        $this->environment = $environment;
        $this->serviceMoviEst = $serviceMoviEst;
        $this->crudService   = $crudService;
        $this->parameterBag   = $parameterBag;
        $this->reportService = $reportService;
        $this->birtStrategy = $birtStrategy;
        
	}

	public function geraRelatorio(DTO\Request\Row $request, DTO\Response $response){
		$row = $request->getRow();

		$sequencial = $this->newCode->getCodeByTableName('MOVIESTOQUE', $row['NRORG']);

		$this->serviceMoviEst->procMovimentaEstoque($row['CDFILIAL'], ' ', ' ', ' ', 5, $row['DTPERINI'],$row['DTPERFIN'],$row['CDPRODINI'],$row['CDPRODFIN'],3,$row['CUSTOS'],$sequencial,null,null,null);

        $nvTotalizacao = $row['NPTOTALIZA']?  $row['NPTOTALIZA'] : 1;

		$relatorio = $this->serviceMoviEst->selectAjusteAnalitico($nvTotalizacao,$row['DTPERINI'],$row['DTPERFIN'],$row['CDPRODINI'],$row['CDPRODFIN'],$sequencial,$row['ORDENA'],$row['CUSTOS']);
        
        
		$totalizador = $this->serviceMoviEst->selectAjusteTotalizador($nvTotalizacao,$row['DTPERINI'],$row['DTPERFIN'],$row['CDPRODINI'],$row['CDPRODFIN'],$sequencial,$row['ORDENA'],$row['CUSTOS'],$row['CDFILIAL']);
		
        $EmissaoAjusteData = [
            'Details'=>$relatorio,
            'Totalizator'=>$totalizador
        ];
        
		$relatorio = new DataSet('EmissaoAjuste',$EmissaoAjusteData);

		$response->addDataset($relatorio);
	}
	
	public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $nrorg = $this->environment->getNrOrgTrab();
        $reportStrategy = array (
            'strategy' => 'BIRT',
            'parameterMapping' => array (
                "SEQUMOVI"     => "P_SEQUMOVI",
                "CDFILIAL"     => "P_CDFILIAL",
                "DTPERINI"     => "P_DTINICIAL",
                "DTPERFIN"     => "P_DTFINAL",
                "NRORG"        => "P_NRORG"
            )
        );

        $row['SEQUMOVI'] = $this->newCode->getCodeByTableName('MOVIESTOQUE', $this->environment->getNrOrgTrab());
        $reportStrategy['parameterMapping']['CUSTOS']        = "P_CUSTO";
        $reportStrategy['parameterMapping']['CDALMOXARIFE']  = "P_CDALMOXARIFE";
        $reportStrategy['parameterMapping']['CDLOCALESTOQ']  = "P_CDLOCALESTOQ";
        $reportStrategy['parameterMapping']['NRLOTEESTOQ']   = "P_NRLOTEESTOQ";
        $reportStrategy['parameterMapping']['NPTOTALIZA']    = "P_NPTOTALIZA";
        $reportStrategy['parameterMapping']['CDPRODINI']     = "P_CDPRODINI";
        $reportStrategy['parameterMapping']['CDPRODFIN']     = "P_CDPRODFIN";
        $reportStrategy['parameterMapping']['ORDENANOME']	 = "P_ORDENANOME";
        $reportStrategy['parameterMapping']['CDFILIAL']		 = "P_CDFILIAL";
        $reportStrategy['parameterMapping']['IDLANCTO']		 = "P_IDLANCTO";
        $reportStrategy['parameterMapping']['NRORG']		 = "P_NRORG";
        $reportName = $row['REPORT_NAME'];

        
        date_default_timezone_set('America/Sao_Paulo');
        $dataEmissao = date('d/m/Y H:i');
        $reportStrategy['parameterMapping']['DTEMISSAO']      = "P_DTEMISSAO";
        $reportStrategy['parameterMapping']['PRODINI']        = "P_PRODINI";
        $reportStrategy['parameterMapping']['PRODFIN']        = "P_PRODFIN";
        $row['P_DTEMISSAO'] = $dataEmissao;
        
        $parametersRow = array();

        foreach ($reportStrategy['parameterMapping'] as $fieldName => $paramName) {
            
            if ($row->has($fieldName)){
                $parametersRow[$paramName] = $row->get($fieldName);
                $row[$paramName] = $row->get($fieldName);
            } 
        }
     
        $parametersRow['P_CDPRODINI'] = $row['CDARVPRODINI'];
        $row['P_CDPRODINI'] = $row['CDARVPRODINI'];
        $parametersRow['P_CDPRODFIN'] = $row['CDARVPRODFIN'];
        $row['P_CDPRODFIN'] = $row['CDARVPRODFIN'];
        $parametersRow['__format'] = $row['__FORMAT'];
        $row['__format'] = $row['__FORMAT'];
        $parametersRow['P_NRORG'] = $nrorg;
        $row['P_NRORG'] = $nrorg;
        $parametersRow['P_IDLANCTO'] = 3;
        $row['P_IDLANCTO'] = 3;
        
        
        $birtURL = $this->reportService->createRemoteReport('BIRT', $reportName, $parametersRow);
        $birtLink = $this->reportService->createRemoteReport('BIRT', $reportName, $parametersRow, true);
        
        $response->addMessage(new DTO\Response\Message($birtURL));
        $response->addMessage(new DTO\Response\Message($birtLink)); 
        
       /* 
        $birtLink = $this->reportService->createRemoteReport('BIRT', $reportName, $parametersRow, true);
        $response->addMessage(new DTO\Response\Message($birtLink));
        
        $birtURL = $this->reportService->createRemoteReport('BIRT', $reportName, $parametersRow, true);
        $birtFile = $this->reportService->createRemoteReport('BIRT', $reportName, $parametersRow);

        $response->addMessage(new DTO\Response\Message($birtURL));
        $response->addMessage(new DTO\Response\Message($birtFile));*/
    }

}
